@@ -37,6 +37,7 @@ int* g_window_border_inner_width;
 int* g_raise_on_focus;
 int* g_snap_gap;
 int* g_smart_window_surroundings;
+int* g_pseudotile_center_threshold;
 unsigned long g_window_border_active_color;
 unsigned long g_window_border_normal_color;
 unsigned long g_window_border_urgent_color;
@@ -65,7 +66,8 @@ static HSClient* create_client() {
     hc->pseudotile = false;
     hc->ewmhrequests = true;
     hc->ewmhnotify = true;
-    hc->sizehints = true;
+    hc->sizehints_floating = true;
+    hc->sizehints_tiling = false;
     return hc;
 }
 
@@ -84,6 +86,7 @@ static void fetch_colors() {
     g_window_border_urgent_color = getcolor(str);
     str = settings_find("window_border_inner_color")->value.s;
     g_window_border_inner_color = getcolor(str);
+    g_pseudotile_center_threshold = &(settings_find("pseudotile_center_threshold")->value.i);
 }
 
 void clientlist_init() {
@@ -117,9 +120,17 @@ static void client_move_to_floatpos(void* key, void* client_void, void* data) {
     }
 }
 
+static void client_show_window(void* key, void* client_void, void* data) {
+    (void)key;
+    (void)data;
+    HSClient* client = client_void;
+    window_show(client->window);
+}
+
 void clientlist_destroy() {
     // move all clients to their original floating position
     g_hash_table_foreach(g_clients, client_move_to_floatpos, NULL);
+    g_hash_table_foreach(g_clients, client_show_window, NULL);
 
     g_hash_table_destroy(g_clients);
     hsobject_unlink_and_destroy(hsobject_root(), g_client_object);
@@ -141,6 +152,25 @@ HSClient* get_client_from_window(Window window) {
     }   \
     while (0);
 
+static void client_attr_tag(void* data, GString* output) {
+    HSClient* client = (HSClient*) data;
+    g_string_append(output, client->tag->display_name->str);
+}
+
+static void client_attr_class(void* data, GString* output) {
+    HSClient* client = (HSClient*) data;
+    GString* ret = window_class_to_g_string(g_display, client->window);
+    g_string_append(output, ret->str);
+    g_string_free(ret, true);
+}
+
+static void client_attr_instance(void* data, GString* output) {
+    HSClient* client = (HSClient*) data;
+    GString* ret = window_instance_to_g_string(g_display, client->window);
+    g_string_append(output, ret->str);
+    g_string_free(ret, true);
+}
+
 static GString* client_attr_fullscreen(HSAttribute* attr) {
     CLIENT_UPDATE_ATTR(client_set_fullscreen, fullscreen);
 }
@@ -153,6 +183,28 @@ static GString* client_attr_urgent(HSAttribute* attr) {
     CLIENT_UPDATE_ATTR(client_set_urgent_force, urgent);
 }
 
+static GString* client_attr_sh_tiling(HSAttribute* attr) {
+    HSClient* client = container_of(attr->value.b, HSClient, sizehints_tiling);
+    if (!is_client_floated(client) && !client->pseudotile) {
+        HSMonitor* mon = find_monitor_with_tag(client->tag);
+        if (mon) {
+            monitor_apply_layout(mon);
+        }
+    }
+    return NULL;
+}
+
+static GString* client_attr_sh_floating(HSAttribute* attr) {
+    HSClient* client = container_of(attr->value.b, HSClient, sizehints_floating);
+    if (!is_client_floated(client) || client->pseudotile) {
+        HSMonitor* mon = find_monitor_with_tag(client->tag);
+        if (mon) {
+            monitor_apply_layout(mon);
+        }
+    }
+    return NULL;
+}
+
 HSClient* manage_client(Window win) {
     if (is_herbstluft_window(g_display, win)) {
         // ignore our own window
@@ -211,24 +263,32 @@ HSClient* manage_client(Window win) {
     client->slice = slice_create_client(client);
     stack_insert_slice(client->tag->stack, client->slice);
     // insert window to the tag
-    frame_insert_window(lookup_frame(client->tag->frame, changes.tree_index->str), win);
+    frame_insert_client(lookup_frame(client->tag->frame, changes.tree_index->str), client);
     client_update_wm_hints(client);
+    updatesizehints(client);
     if (changes.focus) {
         // give focus to window if wanted
         // TODO: make this faster!
         // WARNING: this solution needs O(C + exp(D)) time where W is the count
         // of clients on this tag and D is the depth of the binary layout tree
-        frame_focus_window(client->tag->frame, win);
+        frame_focus_client(client->tag->frame, client);
     }
 
+    client->object.data = client;
+
     HSAttribute attributes[] = {
         ATTRIBUTE_STRING(   "winid",        client->window_str,     ATTR_READ_ONLY),
         ATTRIBUTE_STRING(   "title",        client->title,          ATTR_READ_ONLY),
+        ATTRIBUTE_CUSTOM(   "tag",          client_attr_tag,        ATTR_READ_ONLY),
+        ATTRIBUTE_INT(      "pid",          client->pid,            ATTR_READ_ONLY),
+        ATTRIBUTE_CUSTOM(   "class",        client_attr_class,      ATTR_READ_ONLY),
+        ATTRIBUTE_CUSTOM(   "instance",     client_attr_instance,   ATTR_READ_ONLY),
         ATTRIBUTE_BOOL(     "fullscreen",   client->fullscreen,     client_attr_fullscreen),
         ATTRIBUTE_BOOL(     "pseudotile",   client->pseudotile,     client_attr_pseudotile),
         ATTRIBUTE_BOOL(     "ewmhrequests", client->ewmhrequests,   ATTR_ACCEPT_ALL),
         ATTRIBUTE_BOOL(     "ewmhnotify",   client->ewmhnotify,     ATTR_ACCEPT_ALL),
-        ATTRIBUTE_BOOL(     "sizehints",    client->sizehints,      ATTR_ACCEPT_ALL),
+        ATTRIBUTE_BOOL(     "sizehints_tiling",   client->sizehints_tiling, client_attr_sh_tiling),
+        ATTRIBUTE_BOOL(     "sizehints_floating", client->sizehints_floating, client_attr_sh_floating),
         ATTRIBUTE_BOOL(     "urgent",       client->urgent,         client_attr_urgent),
         ATTRIBUTE_LAST,
     };
@@ -256,6 +316,7 @@ HSClient* manage_client(Window win) {
             monitor_set_tag(get_current_monitor(), client->tag);
         }
     }
+    client_send_configure(client);
 
     client_changes_free_members(&changes);
     grab_client_buttons(client, false);
@@ -272,7 +333,7 @@ void unmanage_client(Window win) {
         mouse_stop_drag();
     }
     // remove from tag
-    frame_remove_window(client->tag->frame, win);
+    frame_remove_client(client->tag->frame, client);
     // ignore events from it
     XSelectInput(g_display, win, 0);
     //XUngrabButton(g_display, AnyButton, AnyModifier, win);
@@ -368,7 +429,7 @@ void window_focus(Window window) {
 
     lastfocus = window;
     /* do some specials for the max layout */
-    bool is_max_layout = frame_focused_window(g_cur_frame) == window
+    bool is_max_layout = frame_focused_client(g_cur_frame) == client
                          && g_cur_frame->content.clients.layout == LAYOUT_MAX
                          && get_current_monitor()->tag->floating == false;
     if (*g_raise_on_focus || is_max_layout) {
@@ -393,7 +454,7 @@ void client_setup_border(HSClient* client, bool focused) {
     }
 }
 
-void client_resize_fullscreen(HSClient* client, HSMonitor* m) {
+static void client_resize_fullscreen(HSClient* client, HSMonitor* m) {
     if (!client || !m) {
         HSDebug("client_resize_fullscreen() got invalid parameters\n");
         return;
@@ -403,7 +464,8 @@ void client_resize_fullscreen(HSClient* client, HSMonitor* m) {
     client->last_border_width = 0;
     XMoveResizeWindow(g_display, client->window,
                       m->rect.x, m->rect.y, m->rect.width, m->rect.height);
-
+    client_send_configure(client);
+    XSync(g_display, False);
 }
 
 void client_raise(HSClient* client) {
@@ -411,7 +473,13 @@ void client_raise(HSClient* client) {
     stack_raise_slide(client->tag->stack, client->slice);
 }
 
-void client_resize(HSClient* client, XRectangle rect, HSFrame* frame) {
+
+void client_resize_tiling(HSClient* client, Rectangle rect, HSFrame* frame) {
+    HSMonitor* m;
+    if (client->fullscreen && (m = find_monitor_with_tag(client->tag))) {
+        client_resize_fullscreen(client, m);
+        return;
+    }
     // ensure minimum size
     if (rect.width < WINDOW_MIN_WIDTH) {
         rect.width = WINDOW_MIN_WIDTH;
@@ -424,38 +492,14 @@ void client_resize(HSClient* client, XRectangle rect, HSFrame* frame) {
         return;
     }
     Window win = client->window;
-    if (client->pseudotile) {
-        XRectangle size = client->float_size;
-        // floating sizes don't include window border, tiling sizes do.
-        // so convert the floating size to a tiling size
-        size.width  += *g_window_border_width * 2;
-        size.height += *g_window_border_width * 2;
-        // ensure size is not larger than rect-tile
-        size.width  = MIN(size.width, rect.width);
-        size.height = MIN(size.height, rect.height);
-
-        // center it
-        rect.x = rect.x + rect.width/2 - size.width/2;
-        rect.y = rect.y + rect.height/2 - size.height/2;
-        rect.width = size.width;
-        rect.height = size.height;
-    }
     int border_width = *g_window_border_width;
     if (*g_smart_window_surroundings && !client->pseudotile
         && (frame->content.clients.count == 1
             || frame->content.clients.layout == LAYOUT_MAX)) {
         border_width = 0;
     }
-    if (RECTANGLE_EQUALS(client->last_size, rect)
-        && client->last_border_width == border_width) {
-        return;
-    }
-    client->last_size = rect;
-    client->last_border_width = border_width;
 
     // apply border width
-    rect.width -= border_width * 2;
-    rect.height -= border_width * 2;
     bool is_max_layout = frame->content.clients.layout != LAYOUT_MAX;
     bool only_one_client = frame->content.clients.count != 1;
     bool smart_surroundings_are_applied = *g_smart_window_surroundings && is_max_layout && only_one_client;
@@ -464,6 +508,36 @@ void client_resize(HSClient* client, XRectangle rect, HSFrame* frame) {
         rect.width -= *g_window_gap;
         rect.height -= *g_window_gap;
     }
+    Rectangle tile = rect;
+    rect.width -= border_width * 2;
+    rect.height -= border_width * 2;
+
+    if (client->pseudotile) {
+        Rectangle size = client->float_size;
+        // ensure size is not larger than the tile
+        rect.width  = MIN(size.width,  tile.width  - 2 * border_width);
+        rect.height = MIN(size.height, tile.height - 2 * border_width);
+    }
+    applysizehints(client, &rect.width, &rect.height);
+    // force it into the tile
+    rect.width = MIN(tile.width - 2*border_width,   rect.width);
+    rect.height = MIN(tile.height - 2*border_width, rect.height);
+    // center the window in the tile
+    // but only if it's relative coordinates would not be too close to the
+    // upper left tile border
+    int threshold = *g_pseudotile_center_threshold;
+    int dx = tile.width/2 - rect.width/2 - border_width;
+    int dy = tile.height/2 - rect.height/2 - border_width;
+    rect.x = tile.x + ((dx < threshold) ? 0 : dx);
+    rect.y = tile.y + ((dy < threshold) ? 0 : dy);
+
+    if (RECTANGLE_EQUALS(client->last_size, rect)
+        && client->last_border_width == border_width) {
+        return;
+    }
+    client->last_border_width = border_width;
+
+    client->last_size = rect;
 
     XWindowChanges changes = {
       .x = rect.x, .y = rect.y, .width = rect.width, .height = rect.height,
@@ -481,30 +555,135 @@ void client_resize(HSClient* client, XRectangle rect, HSFrame* frame) {
                                  g_window_border_inner_color,
                                  current_border_color);
     }
-    //// send new size to client
-    //// WHY SHOULD I? -> faster? only one call?
-    //XConfigureEvent ce;
-    //ce.type = ConfigureNotify;
-    //ce.display = g_display;
-    //ce.event = win;
-    //ce.window = win;
-    //ce.x = rect.x;
-    //ce.y = rect.y;
-    //ce.width = rect.width;
-    //ce.height = rect.height;
-    //ce.border_width = 0;
-    //ce.above = None;
-    //ce.override_redirect = False;
-    //XSendEvent(g_display, win, False, StructureNotifyMask, (XEvent *)&ce);
+    // send new size to client
+    client_send_configure(client);
+    XSync(g_display, False);
 }
 
-void client_resize_tiling(HSClient* client, XRectangle rect, HSFrame* frame) {
-    HSMonitor* m;
-    if (client->fullscreen && (m = find_monitor_with_tag(client->tag))) {
-        client_resize_fullscreen(client, m);
-    } else {
-        client_resize(client, rect, frame);
+
+// from dwm.c
+bool applysizehints(HSClient *c, int *w, int *h) {
+    bool baseismin;
+
+    /* set minimum possible */
+    *w = MAX(1, *w);
+    *h = MAX(1, *h);
+    if(*h < WINDOW_MIN_HEIGHT)
+        *h = WINDOW_MIN_HEIGHT;
+    if(*w < WINDOW_MIN_WIDTH)
+        *w = WINDOW_MIN_WIDTH;
+    bool sizehints = (is_client_floated(c) || c->pseudotile)
+                        ? c->sizehints_floating
+                        : c->sizehints_tiling;
+    if(sizehints) {
+        /* see last two sentences in ICCCM 4.1.2.3 */
+        baseismin = c->basew == c->minw && c->baseh == c->minh;
+        if(!baseismin) { /* temporarily remove base dimensions */
+            *w -= c->basew;
+            *h -= c->baseh;
+        }
+        /* adjust for aspect limits */
+        if(c->mina > 0 && c->maxa > 0) {
+            if(c->maxa < (float)*w / *h)
+                *w = *h * c->maxa + 0.5;
+            else if(c->mina < (float)*h / *w)
+                *h = *w * c->mina + 0.5;
+        }
+        if(baseismin) { /* increment calculation requires this */
+            *w -= c->basew;
+            *h -= c->baseh;
+        }
+        /* adjust for increment value */
+        if(c->incw)
+            *w -= *w % c->incw;
+        if(c->inch)
+            *h -= *h % c->inch;
+        /* restore base dimensions */
+        *w += c->basew;
+        *h += c->baseh;
+        *w = MAX(*w, c->minw);
+        *h = MAX(*h, c->minh);
+        if(c->maxw)
+            *w = MIN(*w, c->maxw);
+        if(c->maxh)
+            *h = MIN(*h, c->maxh);
     }
+    return *w != c->last_size.width || *h != c->last_size.height;
+}
+
+bool applysizehints_xy(HSClient *c, int *x, int *y, int *w, int *h) {
+    return applysizehints(c,w,h) || *x != c->last_size.x
+                                 || *y != c->last_size.y;
+}
+
+// from dwm.c
+void updatesizehints(HSClient *c) {
+    long msize;
+    XSizeHints size;
+
+    if(!XGetWMNormalHints(g_display, c->window, &size, &msize))
+        /* size is uninitialized, ensure that size.flags aren't used */
+        size.flags = PSize;
+    if(size.flags & PBaseSize) {
+        c->basew = size.base_width;
+        c->baseh = size.base_height;
+    }
+    else if(size.flags & PMinSize) {
+        c->basew = size.min_width;
+        c->baseh = size.min_height;
+    } else {
+        c->basew = c->baseh = 0;
+    }
+    if(size.flags & PResizeInc) {
+        c->incw = size.width_inc;
+        c->inch = size.height_inc;
+    }
+    else
+        c->incw = c->inch = 0;
+    if(size.flags & PMaxSize) {
+        c->maxw = size.max_width;
+        c->maxh = size.max_height;
+    } else {
+        c->maxw = c->maxh = 0;
+    }
+    if(size.flags & PMinSize) {
+        c->minw = size.min_width;
+        c->minh = size.min_height;
+    }
+    else if(size.flags & PBaseSize) {
+        c->minw = size.base_width;
+        c->minh = size.base_height;
+    } else {
+        c->minw = c->minh = 0;
+    }
+    if(size.flags & PAspect) {
+        c->mina = (float)size.min_aspect.y / size.min_aspect.x;
+        c->maxa = (float)size.max_aspect.x / size.max_aspect.y;
+    } else {
+        c->maxa = c->mina = 0.0;
+    }
+    //c->isfixed = (c->maxw && c->minw && c->maxh && c->minh
+    //             && c->maxw == c->minw && c->maxh == c->minh);
+}
+
+
+
+
+void client_send_configure(HSClient *c) {
+    XConfigureEvent ce;
+    int bw = c->last_border_width;
+    ce.type = ConfigureNotify;
+    ce.display = g_display;
+    ce.event = c->window;
+    ce.window = c->window;
+    ce.x = c->last_size.x;
+    ce.y = c->last_size.y;
+    ce.width = c->last_size.width;
+    ce.height = c->last_size.height;
+    ce.border_width = bw;
+    ce.above = None;
+    ce.override_redirect = False;
+    XSendEvent(g_display, c->window, False, StructureNotifyMask, (XEvent *)&ce);
 }
 
 void client_resize_floating(HSClient* client, HSMonitor* m) {
@@ -525,29 +704,29 @@ void client_resize_floating(HSClient* client, HSMonitor* m) {
         client->last_border_width = *g_window_border_width;
         border_changed = true;
     }
-    client->last_size = client->float_size;
-    client->last_size.x += m->rect.x + m->pad_left;
-    client->last_size.y += m->rect.y + m->pad_up;
+    Rectangle rect = client->float_size;
+    rect.x += m->rect.x + m->pad_left;
+    rect.y += m->rect.y + m->pad_up;
 
     // ensure position is on monitor
     int space = g_monitor_float_treshold;
-    client->last_size.x =
-        CLAMP(client->last_size.x,
-              m->rect.x + m->pad_left - client->last_size.width + space,
+    rect.x =
+        CLAMP(rect.x,
+              m->rect.x + m->pad_left - rect.width + space,
               m->rect.x + m->rect.width - m->pad_left - m->pad_right - space);
-    client->last_size.y =
-        CLAMP(client->last_size.y,
-              m->rect.y + m->pad_up - client->last_size.height + space,
+    rect.y =
+        CLAMP(rect.y,
+              m->rect.y + m->pad_up - rect.height + space,
               m->rect.y + m->rect.height - m->pad_up - m->pad_down - space);
-    XRectangle rect = client->last_size;
-    // add window border to last_size
-    client->last_size.width += 2 * client->last_border_width;
-    client->last_size.height += 2 * client->last_border_width;
+    if (applysizehints_xy(client, &rect.x, &rect.y, &rect.width, &rect.height)) {
+        XMoveResizeWindow(g_display, client->window,
+            rect.x, rect.y, rect.width, rect.height);
+    }
     if (border_changed) {
         XSetWindowBorderWidth(g_display, client->window, *g_window_border_width);
     }
-    XMoveResizeWindow(g_display, client->window,
-        rect.x, rect.y, rect.width, rect.height);
+    // add window border to last_size
+    client->last_size = rect;
     if (*g_window_border_inner_width > 0
         && *g_window_border_inner_width < *g_window_border_width) {
         unsigned long current_border_color = get_window_border_color(client);
@@ -559,30 +738,39 @@ void client_resize_floating(HSClient* client, HSMonitor* m) {
                                  g_window_border_inner_color,
                                  current_border_color);
     }
+    client_send_configure(client);
+    XSync(g_display, False);
 }
 
-XRectangle client_outer_floating_rect(HSClient* client) {
-    XRectangle rect = client->float_size;
+Rectangle client_outer_floating_rect(HSClient* client) {
+    Rectangle rect = client->float_size;
     rect.width  += *g_window_border_width * 2 + *g_snap_gap;
     rect.height += *g_window_border_width * 2 + *g_snap_gap;
     return rect;
 }
 
-// from dwm.c
-int window_close_current() {
+int close_command(int argc, char** argv, GString* output) {
+    Window win;
+    HSClient* client = NULL;
+    win = string_to_client((argc > 1) ? argv[1] : "", &client);
+    if (win) window_close(win);
+    else return HERBST_INVALID_ARGUMENT;
+    return 0;
+}
+
+bool is_client_floated(HSClient* client) {
+    return client->tag->floating;
+}
+
+void window_close(Window window) {
     XEvent ev;
-    // if there is no focus, then there is nothing to do
-    if (!g_cur_frame) return 0;
-    Window win = frame_focused_window(g_cur_frame);
-    if (!win) return 0;
     ev.type = ClientMessage;
-    ev.xclient.window = win;
+    ev.xclient.window = window;
     ev.xclient.message_type = g_wmatom[WMProtocols];
     ev.xclient.format = 32;
     ev.xclient.data.l[0] = g_wmatom[WMDelete];
     ev.xclient.data.l[1] = CurrentTime;
-    XSendEvent(g_display, win, False, NoEventMask, &ev);
-    return 0;
+    XSendEvent(g_display, window, False, NoEventMask, &ev);
 }
 
 void window_set_visible(Window win, bool visible) {
@@ -617,14 +805,14 @@ void client_set_urgent(HSClient* client, bool state) {
     client_set_urgent_force(client, state);
 }
 
-void client_set_urgent_force(HSClient* client, bool state) {
+static void client_set_urgent_force(HSClient* client, bool state) {
     char winid_str[STRING_BUF_SIZE];
     snprintf(winid_str, STRING_BUF_SIZE, "0x%lx", client->window);
     hook_emit_list("urgent", state ? "on" : "off", winid_str, NULL);
 
     client->urgent = state;
 
-    client_setup_border(client, client->window == frame_focused_window(g_cur_frame));
+    client_setup_border(client, client == frame_focused_client(g_cur_frame));
 
     XWMHints *wmh;
     if(!(wmh = XGetWMHints(g_display, client->window)))
@@ -649,8 +837,8 @@ void client_update_wm_hints(HSClient* client) {
         return;
     }
 
-    Window focused_window = frame_focused_window(g_cur_frame);
-    if ((focused_window == client->window)
+    HSClient* focused_client = frame_focused_client(g_cur_frame);
+    if ((focused_client == client)
         && wmh->flags & XUrgencyHint) {
         // remove urgency hint if window is focused
         wmh->flags &= ~XUrgencyHint;
@@ -661,7 +849,7 @@ void client_update_wm_hints(HSClient* client) {
             client->urgent = newval;
             char winid_str[STRING_BUF_SIZE];
             snprintf(winid_str, STRING_BUF_SIZE, "0x%lx", client->window);
-            client_setup_border(client, focused_window == client->window);
+            client_setup_border(client, focused_client == client);
             hook_emit_list("urgent", client->urgent ? "on":"off", winid_str, NULL);
             tag_set_flags_dirty();
         }
@@ -700,9 +888,7 @@ void client_update_title(HSClient* client) {
 }
 
 HSClient* get_current_client() {
-    Window win = frame_focused_window(g_cur_frame);
-    if (!win) return NULL;
-    return get_client_from_window(win);
+    return frame_focused_client(g_cur_frame);
 }
 
 void client_set_fullscreen(HSClient* client, bool state) {
@@ -780,8 +966,7 @@ void window_update_border(Window window, unsigned long color) {
 }
 
 unsigned long get_window_border_color(HSClient* client) {
-    Window win = client->window;
-    unsigned long current_border_color = (win == frame_focused_window(g_cur_frame)
+    unsigned long current_border_color = (client == frame_focused_client(g_cur_frame)
                                           ? g_window_border_active_color
                                           : g_window_border_normal_color);
     if (client->urgent)
@@ -804,7 +989,8 @@ HSClient* get_urgent_client() {
  *
  * \param   str     Describes the window: "" means the focused one, "urgent"
  *                  resolves to a arbitrary urgent window, "0x..." just
- *                  resolves to the given window.
+ *                  resolves to the given window given its hexadecimal window id,
+ *                  a decimal number its decimal window id.
  * \param   ret_client  The client pointer is stored there if ret_client is
  *                      given and the specified window is managed.
  * \return          The resolved window id is stored there if the according
@@ -813,9 +999,10 @@ HSClient* get_urgent_client() {
 Window string_to_client(char* str, HSClient** ret_client) {
     Window win = 0;
     if (!strcmp(str, "")) {
-        win = frame_focused_window(g_cur_frame);
+        HSClient* client = get_current_client();
+        win = client ? client->window : 0;
         if (ret_client) {
-            *ret_client = get_client_from_window(win);
+            *ret_client = client;
         }
     } else if (!strcmp(str, "urgent")) {
         HSClient* client = get_urgent_client();
@@ -829,6 +1016,10 @@ Window string_to_client(char* str, HSClient** ret_client) {
         if (ret_client) {
             *ret_client = get_client_from_window(win);
         }
+    } else if (1 == sscanf(str, "%lu", (long unsigned int*)&win)) {
+        if (ret_client) {
+            *ret_client = get_client_from_window(win);
+        }
     }
     return win;
 }
