@@ -15,16 +15,17 @@
 
 #include "layout.h"
 #include "object.h"
+#include "utils.h"
 
 struct HSSlice;
 
 typedef struct HSClient {
     Window      window;
     GString*    window_str;     // the window id as a string
-    XRectangle  last_size;      // last size including the window border
+    Rectangle   last_size;      // last size excluding the window border
     int         last_border_width;
     HSTag*      tag;
-    XRectangle  float_size;     // floating size without the window border
+    Rectangle   float_size;     // floating size without the window border
     GString*    title;  // or also called window title; this is never NULL
     bool        urgent;
     bool        fullscreen;
@@ -33,9 +34,14 @@ typedef struct HSClient {
     bool        neverfocus; // do not give the focus via XSetInputFocus
     bool        ewmhrequests; // accept ewmh-requests for this client
     bool        ewmhnotify; // send ewmh-notifications for this client
-    bool        sizehints;  // respect size hints regarding this client
+    bool        sizehints_floating;  // respect size hints regarding this client in floating mode
+    bool        sizehints_tiling;  // respect size hints regarding this client in tiling mode
     bool        dragged;  // if this client is dragged currently
     int         pid;
+    // for size hints
+	float mina, maxa;
+    int basew, baseh, incw, inch, maxw, maxh, minw, minh;
+    // for other modules
     HSObject    object;
     struct HSSlice* slice;
 } HSClient;
@@ -64,18 +70,25 @@ void client_destroy(HSClient* client);
 HSClient* get_client_from_window(Window window);
 HSClient* get_current_client();
 HSClient* get_urgent_client();
-XRectangle client_outer_floating_rect(HSClient* client);
+Rectangle client_outer_floating_rect(HSClient* client);
 
 Window string_to_client(char* str, HSClient** ret_client);
 void client_setup_border(HSClient* client, bool focused);
-void client_resize(HSClient* client, XRectangle rect, HSFrame* frame);
-void client_resize_tiling(HSClient* client, XRectangle rect, HSFrame* frame);
+void client_resize(HSClient* client, Rectangle rect, HSFrame* frame);
+void client_resize_tiling(HSClient* client, Rectangle rect, HSFrame* frame);
 void client_resize_floating(HSClient* client, HSMonitor* m);
+bool is_client_floated(HSClient* client);
 void client_set_urgent(HSClient* client, bool state);
 void client_update_wm_hints(HSClient* client);
 void client_update_title(HSClient* client);
 void client_raise(HSClient* client);
-int window_close_current();
+int close_command(int argc, char** argv, GString* output);
+void window_close(Window window);
+
+void client_send_configure(HSClient *c);
+bool applysizehints(HSClient *c, int *w, int *h);
+bool applysizehints_xy(HSClient *c, int *x, int *y, int *w, int *h);
+void updatesizehints(HSClient *c);
 
 bool client_sendevent(HSClient *client, Atom proto);
 
