@@ -38,7 +38,7 @@ static char* completion_special_winids[]= { "urgent", "", NULL };
 static char* completion_use_index_args[]= { "--skip-visible", NULL };
 static char* completion_cycle_all_args[]= { "--skip-invisible", NULL };
 static char* completion_pm_one[]= { "+1", "-1", NULL };
-static char* completion_split_modes[]= { "horizontal", "vertical", NULL };
+static char* completion_split_modes[]= { "horizontal", "vertical", "left", "right", "top", "bottom", "explode", NULL };
 static char* completion_split_ratios[]= {
     "0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8", "0.9", NULL };
 
@@ -633,7 +633,7 @@ void complete_against_keybinds(int argc, char** argv, int pos, GString* output)
     key_find_binds(needle, output);
 }
 
-bool parameter_expected(int argc, char** argv, int pos) {
+static bool parameter_expected(int argc, char** argv, int pos) {
     if (pos <= 0 || argc < 1) {
         /* no parameter if there is no command */
         return false;
@@ -861,7 +861,7 @@ void complete_sprintf(int argc, char** argv, int position, GString* output) {
     }
 }
 
-bool first_parameter_is_tag(int argc, char** argv, int pos) {
+static bool first_parameter_is_tag(int argc, char** argv, int pos) {
     // only complete if first parameter is a valid tag
     if (argc >= 2 && find_tag(argv[1]) && pos == 2) {
         return true;
@@ -870,7 +870,7 @@ bool first_parameter_is_tag(int argc, char** argv, int pos) {
     }
 }
 
-bool first_parameter_is_flag(int argc, char** argv, int pos) {
+static bool first_parameter_is_flag(int argc, char** argv, int pos) {
     // only complete if first parameter is a flag like -i or -e
     if (argc >= 2 && argv[1][0] == '-' && pos == 2) {
         return true;
@@ -879,7 +879,7 @@ bool first_parameter_is_flag(int argc, char** argv, int pos) {
     }
 }
 
-bool first_parameter_is_writable_attribute(int argc, char** argv, int pos) {
+static bool first_parameter_is_writable_attribute(int argc, char** argv, int pos) {
     GString* dummy = g_string_new("");
     HSAttribute* attr = NULL;
     if (argc >= 2) {
@@ -889,7 +889,7 @@ bool first_parameter_is_writable_attribute(int argc, char** argv, int pos) {
     return attr && attr->on_change != ATTR_READ_ONLY;
 }
 
-bool parameter_expected_offset(int argc, char** argv, int pos, int offset) {
+static bool parameter_expected_offset(int argc, char** argv, int pos, int offset) {
     if (argc < offset || pos < offset) {
         return true;
     }
@@ -900,11 +900,11 @@ bool parameter_expected_offset(int argc, char** argv, int pos, int offset) {
     return parameter_expected(argc - offset, argv + offset, pos - offset);
 }
 
-bool parameter_expected_offset_2(int argc, char** argv, int pos) {
+static bool parameter_expected_offset_2(int argc, char** argv, int pos) {
     return parameter_expected_offset(argc,argv, pos, 2);
 }
 
-bool parameter_expected_offset_3(int argc, char** argv, int pos) {
+static bool parameter_expected_offset_3(int argc, char** argv, int pos) {
     return parameter_expected_offset(argc,argv, pos, 3);
 }
 
