@@ -276,7 +276,7 @@ void ewmh_update_active_window(Window win) {
         XA_WINDOW, 32, PropModeReplace, (unsigned char*)&(win), 1);
 }
 
-bool focus_stealing_allowed(long source) {
+static bool focus_stealing_allowed(long source) {
     if (*g_focus_stealing_prevention) {
         /* only allow it to pagers/taskbars */
         return (source == 2);
@@ -307,7 +307,10 @@ void ewmh_handle_client_message(XEvent* event) {
             // only steal focus it allowed to the current source
             // (i.e.  me->data.l[0] in this case as specified by EWMH)
             if (focus_stealing_allowed(me->data.l[0])) {
-                focus_window(me->window, true, true);
+                HSClient* client = get_client_from_window(me->window);
+                if (client) {
+                    focus_client(client, true, true);
+                }
             }
             break;
 
