@@ -1,4 +1,4 @@
-/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -50,5 +50,23 @@ int         g_verbose;
         } \
     } while(0)
 
+// macro for very slow asserts, which are only executed if DEBUG is defined
+#ifdef DEBUG
+#define slow_assert(X)                                                  \
+    do {                                                                \
+        if (!(X)) {                                                     \
+            fprintf(stderr, "%s:%d: %s: Slow assertion `%s\' failed.",  \
+                    __FILE__, __LINE__, __func__, #X);                  \
+            abort();                                                    \
+        }                                                               \
+    } while (0)
+#else // DEBUG
+#define slow_assert(ignore)((void) 0)
+#endif // DEBUG
+
+// characters that need to be escaped
+// http://pubs.opengroup.org/onlinepubs/009695399/utilities/xcu_chap02.html
+#define ESCAPE_CHARACTERS "|&;<>()$`\\\"\' \t\n"
+
 #endif
 
