@@ -12,7 +12,7 @@
 void ipc_init();
 void ipc_destroy();
 
-void ipc_add_connection(Window window);
+void ipc_add_connection(Window win);
 // returns true if property was received successfully
 bool ipc_handle_connection(Window window);
 bool is_ipc_connectable(Window window);
