@@ -1,4 +1,4 @@
-/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -8,7 +8,7 @@
 
 #include <stdbool.h>
 #include <X11/Xlib.h>
-#include <glib.h>
+#include "glib-backports.h"
 
 #define KEY_COMBI_SEPARATORS "+-"
 
@@ -29,10 +29,14 @@ int keyunbind(int argc, char** argv, GString* output); //removes a keybinding
 void keybinding_free(KeyBinding* binding);
 
 int key_list_binds(int argc, char** argv, GString* output);
+int list_keysyms(int argc, char** argv, GString* output);
 bool key_remove_bind_with_keysym(unsigned int modifiers, KeySym sym);
 void key_remove_all_binds();
 GString* keybinding_to_g_string(KeyBinding* binding);
 void key_find_binds(char* needle, GString* output);
+void complete_against_modifiers(char* needle, char seperator,
+                                char* prefix, GString* output);
+void complete_against_keysyms(char* needle, char* prefix, GString* output);
 void regrab_keys();
 void grab_keybind(KeyBinding* binding, void* useless_pointer);
 void update_numlockmask();
