@@ -6,6 +6,7 @@
 #include "clientlist.h"
 #include "globals.h"
 #include "utils.h"
+#include "x11-utils.h"
 #include "hook.h"
 #include "ewmh.h"
 #include "ipc-protocol.h"
@@ -35,6 +36,7 @@ int* g_frame_border_inner_width;
 int* g_always_show_frame;
 int* g_default_frame_layout;
 int* g_frame_bg_transparent;
+int* g_frame_transparent_width;
 int* g_direction_external_only;
 int* g_gapless_grid;
 int* g_smart_frame_surroundings;
@@ -71,6 +73,7 @@ static void fetch_frame_colors() {
     g_frame_border_inner_width = &(settings_find("frame_border_inner_width")->value.i);
     g_always_show_frame = &(settings_find("always_show_frame")->value.i);
     g_frame_bg_transparent = &(settings_find("frame_bg_transparent")->value.i);
+    g_frame_transparent_width = &(settings_find("frame_transparent_width")->value.i);
     g_default_frame_layout = &(settings_find("default_frame_layout")->value.i);
     g_direction_external_only = &(settings_find("default_direction_external_only")->value.i);
     g_gapless_grid = &(settings_find("gapless_grid")->value.i);
@@ -150,20 +153,20 @@ HSFrame* frame_create_empty(HSFrame* parent, HSTag* parenttag) {
     return frame;
 }
 
-void frame_insert_window(HSFrame* frame, Window window) {
+void frame_insert_client(HSFrame* frame, struct HSClient* client) {
     if (frame->type == TYPE_CLIENTS) {
         // insert it here
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         // append it to buf
         size_t count = frame->content.clients.count;
         count++;
         // insert it after the selection
         int index = frame->content.clients.selection + 1;
         index = CLAMP(index, 0, count - 1);
-        buf = g_renew(Window, buf, count);
+        buf = g_renew(HSClient*, buf, count);
         // shift other windows to the back to insert the new one at index
         memmove(buf + index + 1, buf + index, sizeof(*buf) * (count - index - 1));
-        buf[index] = window;
+        buf[index] = client;
         // write results back
         frame->content.clients.count = count;
         frame->content.clients.buf = buf;
@@ -171,11 +174,11 @@ void frame_insert_window(HSFrame* frame, Window window) {
         if (g_cur_frame == frame
             && frame->content.clients.selection >= (count-1)) {
             frame->content.clients.selection = count - 1;
-            window_focus(window);
+            window_focus(client->window);
         }
     } else { /* frame->type == TYPE_FRAMES */
         HSLayout* layout = &frame->content.layout;
-        frame_insert_window((layout->selection == 0)? layout->a : layout->b, window);
+        frame_insert_client((layout->selection == 0)? layout->a : layout->b, client);
     }
 }
 
@@ -205,37 +208,37 @@ HSFrame* lookup_frame(HSFrame* root, char *index) {
     return lookup_frame(new_root, new_index);
 }
 
-HSFrame* find_frame_with_window(HSFrame* frame, Window window) {
+HSFrame* find_frame_with_client(HSFrame* frame, struct HSClient* client) {
     if (frame->type == TYPE_CLIENTS) {
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         for (int i = 0; i < count; i++) {
-            if (buf[i] == window) {
+            if (buf[i] == client) {
                 return frame;
             }
         }
         return NULL;
     } else { /* frame->type == TYPE_FRAMES */
-        HSFrame* found = find_frame_with_window(frame->content.layout.a, window);
+        HSFrame* found = find_frame_with_client(frame->content.layout.a, client);
         if (!found) {
-            found = find_frame_with_window(frame->content.layout.b, window);
+            found = find_frame_with_client(frame->content.layout.b, client);
         }
         return found;
     }
 }
 
-bool frame_remove_window(HSFrame* frame, Window window) {
+bool frame_remove_client(HSFrame* frame, HSClient* client) {
     if (frame->type == TYPE_CLIENTS) {
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         int i;
         for (i = 0; i < count; i++) {
-            if (buf[i] == window) {
+            if (buf[i] == client) {
                 // if window was found
                 // them remove it
-                memmove(buf+i, buf+i+1, sizeof(Window)*(count - i - 1));
+                memmove(buf+i, buf+i+1, sizeof(buf[0])*(count - i - 1));
                 count--;
-                buf = g_renew(Window, buf, count);
+                buf = g_renew(HSClient*, buf, count);
                 frame->content.clients.buf = buf;
                 frame->content.clients.count = count;
                 // find out new selection
@@ -252,24 +255,24 @@ bool frame_remove_window(HSFrame* frame, Window window) {
         }
         return false;
     } else { /* frame->type == TYPE_FRAMES */
-        bool found = frame_remove_window(frame->content.layout.a, window);
-        found = found || frame_remove_window(frame->content.layout.b, window);
+        bool found = frame_remove_client(frame->content.layout.a, client);
+        found = found || frame_remove_client(frame->content.layout.b, client);
         return found;
     }
 }
 
-void frame_destroy(HSFrame* frame, Window** buf, size_t* count) {
+void frame_destroy(HSFrame* frame, HSClient*** buf, size_t* count) {
     if (frame->type == TYPE_CLIENTS) {
         *buf = frame->content.clients.buf;
         *count = frame->content.clients.count;
     } else { /* frame->type == TYPE_FRAMES */
         size_t c1, c2;
-        Window *buf1, *buf2;
+        HSClient **buf1, **buf2;
         frame_destroy(frame->content.layout.a, &buf1, &c1);
         frame_destroy(frame->content.layout.b, &buf2, &c2);
         // append buf2 to buf1
-        buf1 = g_renew(Window, buf1, c1 + c2);
-        memcpy(buf1+c1, buf2, sizeof(Window) * c2);
+        buf1 = g_renew(HSClient*, buf1, c1 + c2);
+        memcpy(buf1+c1, buf2, sizeof(buf1[0]) * c2);
         // free unused things
         g_free(buf2);
         // return;
@@ -290,12 +293,12 @@ void dump_frame_tree(HSFrame* frame, GString* output) {
             LAYOUT_DUMP_WHITESPACES[0],
             g_layout_names[frame->content.clients.layout],
             frame->content.clients.selection);
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t i, count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
             g_string_append_printf(output, "%c0x%lx",
                 LAYOUT_DUMP_WHITESPACES[0],
-                buf[i]);
+                buf[i]->window);
         }
         g_string_append_c(output, LAYOUT_DUMP_BRACKETS[1]);
     } else {
@@ -424,16 +427,16 @@ char* load_frame_tree(HSFrame* frame, char* description, GString* errormsg) {
         // ensure that it is a client frame
         if (frame->type == TYPE_FRAMES) {
             // remove childs
-            Window *buf1, *buf2;
+            HSClient **buf1, **buf2;
             size_t count1, count2;
             frame_destroy(frame->content.layout.a, &buf1, &count1);
             frame_destroy(frame->content.layout.b, &buf2, &count2);
 
             // merge bufs
             size_t count = count1 + count2;
-            Window* buf = g_new(Window, count);
-            memcpy(buf,             buf1, sizeof(Window) * count1);
-            memcpy(buf + count1,    buf2, sizeof(Window) * count2);
+            HSClient** buf = g_new(HSClient*, count);
+            memcpy(buf,             buf1, sizeof(buf[0]) * count1);
+            memcpy(buf + count1,    buf2, sizeof(buf[0]) * count2);
             g_free(buf1);
             g_free(buf2);
 
@@ -470,7 +473,7 @@ char* load_frame_tree(HSFrame* frame, char* description, GString* errormsg) {
 
             // remove window from old tag
             HSMonitor* clientmonitor = find_monitor_with_tag(client->tag);
-            if (!frame_remove_window(client->tag->frame, win)) {
+            if (!frame_remove_client(client->tag->frame, client)) {
                 g_warning("window %lx was not found on tag %s\n",
                     win, client->tag->name->str);
             }
@@ -480,14 +483,14 @@ char* load_frame_tree(HSFrame* frame, char* description, GString* errormsg) {
             stack_remove_slice(client->tag->stack, client->slice);
 
             // insert it to buf
-            Window* buf = frame->content.clients.buf;
+            HSClient** buf = frame->content.clients.buf;
             size_t count = frame->content.clients.count;
             count++;
             index = CLAMP(index, 0, count - 1);
-            buf = g_renew(Window, buf, count);
+            buf = g_renew(HSClient*, buf, count);
             memmove(buf + index + 1, buf + index,
-                    sizeof(Window) * (count - index - 1));
-            buf[index] = win;
+                    sizeof(buf[0]) * (count - index - 1));
+            buf[index] = client;
             frame->content.clients.buf = buf;
             frame->content.clients.count = count;
 
@@ -549,10 +552,10 @@ static void frame_append_caption(HSTree tree, GString* output) {
         // list of clients
         g_string_append_printf(output, "%s:",
             g_layout_names[frame->content.clients.layout]);
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t i, count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            g_string_append_printf(output, " 0x%lx", buf[i]);
+            g_string_append_printf(output, " 0x%lx", buf[i]->window);
         }
         if (g_cur_frame == frame) {
             g_string_append(output, " [FOCUS]");
@@ -605,12 +608,12 @@ void frame_apply_floating_layout(HSFrame* frame, HSMonitor* m) {
     } else {
         /* if(frame->type == TYPE_CLIENTS) */
         frame_set_visible(frame, false);
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         size_t selection = frame->content.clients.selection;
         /* border color */
         for (int i = 0; i < count; i++) {
-            HSClient* client = get_client_from_window(buf[i]);
+            HSClient* client = buf[i];
             client_setup_border(client, (g_cur_frame == frame) && (i == selection));
             client_resize_floating(client, m);
         }
@@ -672,11 +675,11 @@ int frame_current_set_client_layout(int argc, char** argv, GString* output) {
     return 0;
 }
 
-void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vertical) {
-    Window* buf = frame->content.clients.buf;
+void frame_apply_client_layout_linear(HSFrame* frame, Rectangle rect, bool vertical) {
+    HSClient** buf = frame->content.clients.buf;
     size_t count = frame->content.clients.count;
     int selection = frame->content.clients.selection;
-    XRectangle cur = rect;
+    Rectangle cur = rect;
     int last_step_y;
     int last_step_x;
     int step_y;
@@ -697,7 +700,7 @@ void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vert
         step_x = cur.width;
     }
     for (int i = 0; i < count; i++) {
-        HSClient* client = get_client_from_window(buf[i]);
+        HSClient* client = buf[i];
         // add the space, if count does not divide frameheight without remainder
         cur.height += (i == count-1) ? last_step_y : 0;
         cur.width += (i == count-1) ? last_step_x : 0;
@@ -708,20 +711,20 @@ void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vert
     }
 }
 
-void frame_apply_client_layout_horizontal(HSFrame* frame, XRectangle rect) {
+void frame_apply_client_layout_horizontal(HSFrame* frame, Rectangle rect) {
     frame_apply_client_layout_linear(frame, rect, false);
 }
 
-void frame_apply_client_layout_vertical(HSFrame* frame, XRectangle rect) {
+void frame_apply_client_layout_vertical(HSFrame* frame, Rectangle rect) {
     frame_apply_client_layout_linear(frame, rect, true);
 }
 
-void frame_apply_client_layout_max(HSFrame* frame, XRectangle rect) {
-    Window* buf = frame->content.clients.buf;
+void frame_apply_client_layout_max(HSFrame* frame, Rectangle rect) {
+    HSClient** buf = frame->content.clients.buf;
     size_t count = frame->content.clients.count;
     int selection = frame->content.clients.selection;
     for (int i = 0; i < count; i++) {
-        HSClient* client = get_client_from_window(buf[i]);
+        HSClient* client = buf[i];
         client_setup_border(client, (g_cur_frame == frame) && (i == selection));
         client_resize_tiling(client, rect, frame);
         if (i == selection) {
@@ -743,8 +746,8 @@ void frame_layout_grid_get_size(size_t count, int* res_rows, int* res_cols) {
     }
 }
 
-void frame_apply_client_layout_grid(HSFrame* frame, XRectangle rect) {
-    Window* buf = frame->content.clients.buf;
+void frame_apply_client_layout_grid(HSFrame* frame, Rectangle rect) {
+    HSClient** buf = frame->content.clients.buf;
     size_t count = frame->content.clients.count;
     int selection = frame->content.clients.selection;
     if (count == 0) {
@@ -756,7 +759,7 @@ void frame_apply_client_layout_grid(HSFrame* frame, XRectangle rect) {
     int width = rect.width / cols;
     int height = rect.height / rows;
     int i = 0;
-    XRectangle cur = rect; // current rectangle
+    Rectangle cur = rect; // current rectangle
     for (int r = 0; r < rows; r++) {
         // reset to left
         cur.x = rect.x;
@@ -777,7 +780,7 @@ void frame_apply_client_layout_grid(HSFrame* frame, XRectangle rect) {
             }
 
             // apply size
-            HSClient* client = get_client_from_window(buf[i]);
+            HSClient* client = buf[i];
             client_setup_border(client, (g_cur_frame == frame) && (i == selection));
             client_resize_tiling(client, cur, frame);
             cur.x += width;
@@ -788,7 +791,8 @@ void frame_apply_client_layout_grid(HSFrame* frame, XRectangle rect) {
 
 }
 
-void frame_apply_layout(HSFrame* frame, XRectangle rect) {
+void frame_apply_layout(HSFrame* frame, Rectangle rect) {
+    frame->last_rect = rect;
     if (frame->type == TYPE_CLIENTS) {
         size_t count = frame->content.clients.count;
         if (!*g_smart_frame_surroundings || frame->parent) {
@@ -808,28 +812,30 @@ void frame_apply_layout(HSFrame* frame, XRectangle rect) {
         }
         unsigned long border_color = g_frame_border_normal_color;
         unsigned long bg_color = g_frame_bg_normal_color;
+        int bw = *g_frame_border_width;
         if (g_cur_frame == frame) {
             border_color = g_frame_border_active_color;
             bg_color = g_frame_bg_active_color;
         }
-        if (!*g_smart_frame_surroundings || frame->parent) {
-            XSetWindowBorderWidth(g_display, frame->window, *g_frame_border_width);
-            XMoveResizeWindow(g_display, frame->window,
-                              rect.x - *g_frame_border_width,
-                              rect.y - *g_frame_border_width,
-                              rect.width, rect.height);
-        } else {
-            XSetWindowBorderWidth(g_display, frame->window, 0);
-            XMoveResizeWindow(g_display, frame->window, rect.x, rect.y, rect.width, rect.height);
+        if (*g_smart_frame_surroundings && !frame->parent) {
+            bw = 0;
         }
+        XSetWindowBorderWidth(g_display, frame->window, bw);
+        XMoveResizeWindow(g_display, frame->window,
+                          rect.x - bw,
+                          rect.y - bw,
+                          rect.width, rect.height);
 
         frame_update_border(frame->window, border_color);
 
-        if (*g_frame_bg_transparent) {
-            XSetWindowBackgroundPixmap(g_display, frame->window, ParentRelative);
-        } else {
-            XSetWindowBackground(g_display, frame->window, bg_color);
+        XSetWindowBackground(g_display, frame->window, bg_color);
+        if (*g_frame_bg_transparent) { // != ) {
+            window_cut_rect_hole(frame->window, rect.width, rect.height,
+                                 *g_frame_transparent_width);
+        } else if (frame->window_transparent) {
+            window_make_intransparent(frame->window, rect.width, rect.height);
         }
+        frame->window_transparent = *g_frame_bg_transparent;
         if (g_cur_frame == frame) {
             ewmh_set_window_opacity(frame->window, g_frame_active_opacity/100.0);
         } else {
@@ -867,8 +873,8 @@ void frame_apply_layout(HSFrame* frame, XRectangle rect) {
         }
     } else { /* frame->type == TYPE_FRAMES */
         HSLayout* layout = &frame->content.layout;
-        XRectangle first = rect;
-        XRectangle second = rect;
+        Rectangle first = rect;
+        Rectangle second = rect;
         if (layout->align == ALIGN_VERTICAL) {
             first.height = (rect.height * layout->fraction) / FRACTION_UNIT;
             second.y += first.height;
@@ -929,8 +935,14 @@ int frame_current_bring(int argc, char** argv, GString* output) {
         }
         return HERBST_INVALID_ARGUMENT;
     }
-    tag_move_client(client, get_current_monitor()->tag);
-    focus_window(client->window, false, false);
+    HSTag* tag = get_current_monitor()->tag;
+    tag_move_client(client, tag);
+    HSFrame* frame = find_frame_with_client(tag->frame, client);
+    if (frame != g_cur_frame) {
+        frame_remove_client(frame, client);
+        frame_insert_client(g_cur_frame, client);
+    }
+    focus_client(client, false, false);
     return 0;
 }
 
@@ -951,7 +963,7 @@ int frame_current_set_selection(int argc, char** argv) {
         index = frame->content.clients.count - 1;
     }
     frame->content.clients.selection = index;
-    Window window = frame->content.clients.buf[index];
+    Window window = frame->content.clients.buf[index]->window;
     window_focus(window);
     return 0;
 }
@@ -976,14 +988,14 @@ int frame_current_cycle_selection(int argc, char** argv) {
     index += count;
     index %= count;
     frame->content.clients.selection = index;
-    Window window = frame->content.clients.buf[index];
+    Window window = frame->content.clients.buf[index]->window;
     window_focus(window);
     return 0;
 }
 
 int cycle_all_command(int argc, char** argv) {
     int delta = 1;
-    int skip_invisible = false;
+    bool skip_invisible = false;
     if (argc >= 2) {
         if (!strcmp(argv[1], "--skip-invisible")) {
             skip_invisible = true;
@@ -1003,28 +1015,62 @@ int cycle_all_command(int argc, char** argv) {
     int index = frame->content.clients.selection;
     bool change_frame = false;
     int direction;
-    int other_direction;
     int new_window_index; // tells where to start in a new frame
     if (delta > 0 && (index + 1) >= frame->content.clients.count) {
         // change selection from 0 to 1
         direction = 1;
-        other_direction = 0;
         change_frame = true;
         new_window_index = 0; // focus first window in in a frame
     }
     if (delta < 0 && index == 0) {
         // change selection from 1 to 0
         direction = 0;
-        other_direction = 1;
         change_frame = true;
         new_window_index = -1; // focus last window in in a frame
     }
     if (skip_invisible && frame->content.clients.layout == LAYOUT_MAX) {
         direction = (delta > 0) ? 1 : 0;
-        other_direction = 1 - direction;
         change_frame = true;
     }
     if (change_frame) {
+        cycle_frame(direction, new_window_index, skip_invisible);
+    } else {
+        // only change the selection within one frame
+        index += delta;
+        // ensure it is a valid index
+        index %= frame->content.clients.count;
+        index += frame->content.clients.count;
+        index %= frame->content.clients.count;
+        frame->content.clients.selection = index;
+    }
+    HSClient* c = frame_focused_client(g_cur_frame);
+    if (c) {
+        client_raise(c);
+    }
+    monitor_apply_layout(get_current_monitor());
+    return 0;
+}
+
+int cycle_frame_command(int argc, char** argv) {
+    int delta = 1;
+    if (argc >= 2) {
+        delta = atoi(argv[1]);
+    }
+    if (delta == 0) return 0;
+    int direction = (delta > 0) ? 1 : 0;
+    cycle_frame(direction, -2, false);
+    return 0;
+}
+
+// cycle_frame:
+//  direction: 1 for forward cycling, 0 for backwards cycling
+//  new_window_index: which index in the new frame should be focused, -2 does
+//  not change the window selection in the new frame
+//  skip_invisible: if set, don't touch the selection in frames in max layout
+void cycle_frame(int direction, int new_window_index, bool skip_invisible) {
+    HSFrame* frame = frame_current_selection();
+    int other_direction = 1 - direction;
+    if (true) {
         HSFrame* top_frame;
         //   these things can be visualized easily for direction = 1
         /*
@@ -1094,7 +1140,7 @@ int cycle_all_command(int argc, char** argv) {
          *   / \   / \
          *  .   . .   .
          */
-        // and then to the left (i.e. find first leave
+        // and then to the left (i.e. find first leaf)
         while (frame->type == TYPE_FRAMES) {
             // then go deeper, with the other direction
             frame->content.layout.selection = other_direction;
@@ -1111,7 +1157,7 @@ int cycle_all_command(int argc, char** argv) {
          */
         // now we reached the next client containing frame
 
-        if (frame->content.clients.count > 0) {
+        if (new_window_index != -2 && frame->content.clients.count > 0) {
             if (!skip_invisible
                 || frame->content.clients.layout != LAYOUT_MAX)
             {
@@ -1127,24 +1173,9 @@ int cycle_all_command(int argc, char** argv) {
         // all changes were made below top_frame
         frame_focus_recursive(top_frame);
 
-    } else {
-        // only change the selection within one frame
-        index += delta;
-        // ensure it is a valid index
-        index %= frame->content.clients.count;
-        index += frame->content.clients.count;
-        index %= frame->content.clients.count;
-        frame->content.clients.selection = index;
-    }
-    Window w = frame_focused_window(g_cur_frame);
-    if (w) {
-        HSClient* c = get_client_from_window(w);
-        if (c) {
-            client_raise(c);
-        }
     }
     monitor_apply_layout(get_current_monitor());
-    return 0;
+    return;
 }
 
 
@@ -1162,10 +1193,10 @@ int frame_split_count_to_root(HSFrame* frame, int align) {
     return height;
 }
 
-void frame_split(HSFrame* frame, int align, int fraction) {
+bool frame_split(HSFrame* frame, int align, int fraction) {
     if (frame_split_count_to_root(frame, align) > HERBST_MAX_TREE_HEIGHT) {
         // do nothing if tree would be to large
-        return;
+        return false;
     }
     // ensure fraction is allowed
     fraction = CLAMP(fraction,
@@ -1183,29 +1214,117 @@ void frame_split(HSFrame* frame, int align, int fraction) {
     frame->content.layout.b = second;
     frame->content.layout.selection = 0;
     frame->content.layout.fraction = fraction;
+    return true;
 }
 
 int frame_split_command(int argc, char** argv, GString* output) {
-    // usage: split h|v FRACTION
-    if (argc < 3) {
+    // usage: split t|b|l|r|h|v FRACTION
+    if (argc < 2) {
         return HERBST_NEED_MORE_ARGS;
     }
-    int align = ALIGN_VERTICAL;
-    if (argv[1][0] == 'h') {
-        align = ALIGN_HORIZONTAL;
-    } else if (argv[1][0] == 'v') {
-        align = ALIGN_VERTICAL;
+    int align = -1;
+    bool userDefinedFraction = true;
+    char* strFraction = "0.5";
+    if (argc < 3) {
+        userDefinedFraction = false;
     } else {
+        strFraction = argv[2];
+    }
+    bool frameToFirst = true;
+    int fraction = FRACTION_UNIT* CLAMP(atof(strFraction),
+                                        0.0 + FRAME_MIN_FRACTION,
+                                        1.0 - FRAME_MIN_FRACTION);
+    int selection = 0;
+    int lh = g_cur_frame->last_rect.height;
+    int lw = g_cur_frame->last_rect.width;
+    int align_auto = (lw > lh) ? ALIGN_HORIZONTAL : ALIGN_VERTICAL;
+    struct {
+        char* name;
+        int align;
+        bool frameToFirst;  // if former frame moves to first child
+        int selection;      // which child to select after the split
+    } splitModes[] = {
+        { "top",        ALIGN_VERTICAL,     false,  1   },
+        { "bottom",     ALIGN_VERTICAL,     true,   0   },
+        { "vertical",   ALIGN_VERTICAL,     true,   0   },
+        { "right",      ALIGN_HORIZONTAL,   true,   0   },
+        { "horizontal", ALIGN_HORIZONTAL,   true,   0   },
+        { "left",       ALIGN_HORIZONTAL,   false,  1   },
+        { "explode",    ALIGN_EXPLODE,      true,   0   },
+    };
+    for (int i = 0; i < LENGTH(splitModes); i++) {
+        if (splitModes[i].name[0] == argv[1][0]) {
+            align           = splitModes[i].align;
+            frameToFirst    = splitModes[i].frameToFirst;
+            selection       = splitModes[i].selection;
+            break;
+        }
+    }
+    if (align < 0) {
         g_string_append_printf(output,
             "%s: Invalid alignment \"%s\"\n", argv[0], argv[1]);
         return HERBST_INVALID_ARGUMENT;
     }
-    int fraction = FRACTION_UNIT* CLAMP(atof(argv[2]),
-                                        0.0 + FRAME_MIN_FRACTION,
-                                        1.0 - FRAME_MIN_FRACTION);
     HSFrame* frame = frame_current_selection();
     if (!frame) return 0; // nothing to do
-    frame_split(frame, align, fraction);
+    bool exploding = align == ALIGN_EXPLODE;
+    int layout = frame->content.clients.layout;
+    int windowcount = frame->content.clients.count;
+    if (exploding) {
+        if (windowcount <= 1) {
+            align = align_auto;
+        } else if (layout == LAYOUT_MAX) {
+            align = align_auto;
+        } else if (layout == LAYOUT_GRID && windowcount == 2) {
+            align = ALIGN_HORIZONTAL;
+        } else if (layout == LAYOUT_HORIZONTAL) {
+            align = ALIGN_HORIZONTAL;
+        } else {
+            align = ALIGN_VERTICAL;
+        }
+        int layout = frame->content.clients.layout;
+        size_t count1 = frame->content.clients.count;
+        size_t nc1 = (count1 + 1) / 2;      // new count for the first frame
+        if ((layout == LAYOUT_HORIZONTAL || layout == LAYOUT_VERTICAL)
+            && !userDefinedFraction && count1 > 1) {
+            fraction = (nc1 * FRACTION_UNIT) / count1;
+        }
+    }
+    if (!frame_split(frame, align, fraction)) {
+        return 0;
+    }
+    if (exploding) {
+        // move second half of the window buf to second frame
+        size_t count1 = frame->content.layout.a->content.clients.count;
+        size_t count2 = frame->content.layout.b->content.clients.count;
+        // assert: count2 == 0
+        size_t nc1 = (count1 + 1) / 2;      // new count for the first frame
+        size_t nc2 = count1 - nc1 + count2; // new count for the 2nd frame
+        HSFrame* child1 = frame->content.layout.a;
+        HSFrame* child2 = frame->content.layout.b;
+        HSClient*** buf1 = &child1->content.clients.buf;
+        HSClient*** buf2 = &child2->content.clients.buf;
+        *buf2 = g_renew(HSClient*, *buf2, nc2);
+        memcpy(*buf2 + count2, *buf1 + nc1, (nc2 - count2) * sizeof(**buf2));
+        *buf1 = g_renew(HSClient*, *buf1, nc1);
+        child1->content.clients.count = nc1;
+        child2->content.clients.count = nc2;
+        child2->content.clients.layout = child1->content.clients.layout;
+        if (child1->content.clients.selection >= nc1 && nc1 > 0) {
+            child2->content.clients.selection =
+                child1->content.clients.selection - nc1 + count2;
+            child1->content.clients.selection = nc1 - 1;
+            selection = 1;
+        } else {
+            selection = 0;
+        }
+
+    } else if (!frameToFirst) {
+        HSFrame* emptyFrame = frame->content.layout.b;
+        frame->content.layout.b = frame->content.layout.a;
+        frame->content.layout.a = emptyFrame;
+    }
+    frame->content.layout.selection = selection;
     // reset focus
     g_cur_frame = frame_current_selection();
     // redraw monitor
@@ -1432,9 +1551,9 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
     if (!external_only &&
         (index = frame_inner_neighbour_index(g_cur_frame, direction)) != -1) {
         int selection = g_cur_frame->content.clients.selection;
-        Window* buf = g_cur_frame->content.clients.buf;
+        HSClient** buf = g_cur_frame->content.clients.buf;
         // if internal neighbour was found, then swap
-        Window tmp = buf[selection];
+        HSClient* tmp = buf[selection];
         buf[selection] = buf[index];
         buf[index] = tmp;
 
@@ -1443,11 +1562,11 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
         monitor_apply_layout(get_current_monitor());
     } else {
         HSFrame* neighbour = frame_neighbour(g_cur_frame, direction);
-        Window win = frame_focused_window(g_cur_frame);
-        if (win && neighbour != NULL) { // if neighbour was found
+        HSClient* client = frame_focused_client(g_cur_frame);
+        if (client && neighbour != NULL) { // if neighbour was found
             // move window to neighbour
-            frame_remove_window(g_cur_frame, win);
-            frame_insert_window(neighbour, win);
+            frame_remove_client(g_cur_frame, client);
+            frame_insert_client(neighbour, client);
 
             // change selection in parent
             HSFrame* parent = neighbour->parent;
@@ -1458,12 +1577,12 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
             HSFrame* frame = g_cur_frame;
             assert(frame);
             int i;
-            Window* buf = frame->content.clients.buf;
+            HSClient** buf = frame->content.clients.buf;
             size_t count = frame->content.clients.count;
             for (i = 0; i < count; i++) {
-                if (buf[i] == win) {
+                if (buf[i] == client) {
                     frame->content.clients.selection = i;
-                    window_focus(buf[i]);
+                    window_focus(buf[i]->window);
                     break;
                 }
             }
@@ -1483,11 +1602,11 @@ void frame_unfocus() {
     //XSetInputFocus(g_display, g_root, RevertToPointerRoot, CurrentTime);
 }
 
-Window frame_focused_window(HSFrame* frame) {
+HSClient* frame_focused_client(HSFrame* frame) {
     if (!frame) {
-        return (Window)0;
+        return NULL;
     }
-    // follow the selection to a leave
+    // follow the selection to a leaf
     while (frame->type == TYPE_FRAMES) {
         frame = (frame->content.layout.selection == 0) ?
                 frame->content.layout.a :
@@ -1497,24 +1616,24 @@ Window frame_focused_window(HSFrame* frame) {
         int selection = frame->content.clients.selection;
         return frame->content.clients.buf[selection];
     } // else, if there are no windows
-    return (Window)0;
+    return NULL;
 }
 
 // try to focus window in frame
 // it does not require anything from the frame. it may be infocused or even
 // hidden.
 // returns true if win was found and focused, else returns false
-bool frame_focus_window(HSFrame* frame, Window win) {
+bool frame_focus_client(HSFrame* frame, HSClient* client) {
     if (!frame) {
         return false;
     }
     if (frame->type == TYPE_CLIENTS) {
         int i;
         size_t count = frame->content.clients.count;
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         // search for win in buf
         for (i = 0; i < count; i++) {
-            if (buf[i] == win) {
+            if (buf[i] == client) {
                 // if found, set focus to it
                 frame->content.clients.selection = i;
                 return true;
@@ -1524,13 +1643,13 @@ bool frame_focus_window(HSFrame* frame, Window win) {
     } else {
         // type == TYPE_FRAMES
         // search in subframes
-        bool found = frame_focus_window(frame->content.layout.a, win);
+        bool found = frame_focus_client(frame->content.layout.a, client);
         if (found) {
             // set selection to first frame
             frame->content.layout.selection = 0;
             return true;
         }
-        found = frame_focus_window(frame->content.layout.b, win);
+        found = frame_focus_client(frame->content.layout.b, client);
         if (found) {
             // set selection to second frame
             frame->content.layout.selection = 1;
@@ -1544,8 +1663,7 @@ bool frame_focus_window(HSFrame* frame, Window win) {
 // switch_tag tells, whether to switch tag to focus to window
 // switch_monitor tells, whether to switch monitor to focus to window
 // returns if window was focused or not
-bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
-    HSClient* client = get_client_from_window(win);
+bool focus_client(struct HSClient* client, bool switch_tag, bool switch_monitor) {
     if (!client) {
         // client is not managed
         return false;
@@ -1583,7 +1701,7 @@ bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
     }
     // now the right tag is visible
     // now focus it
-    bool found = frame_focus_window(tag->frame, win);
+    bool found = frame_focus_client(tag->frame, client);
     frame_focus_recursive(tag->frame);
     monitor_apply_layout(cur_mon);
     monitors_unlock();
@@ -1591,7 +1709,7 @@ bool focus_window(Window win, bool switch_tag, bool switch_monitor) {
 }
 
 int frame_focus_recursive(HSFrame* frame) {
-    // follow the selection to a leave
+    // follow the selection to a leaf
     while (frame->type == TYPE_FRAMES) {
         frame = (frame->content.layout.selection == 0) ?
                 frame->content.layout.a :
@@ -1601,7 +1719,7 @@ int frame_focus_recursive(HSFrame* frame) {
     frame_unfocus();
     if (frame->content.clients.count) {
         int selection = frame->content.clients.selection;
-        window_focus(frame->content.clients.buf[selection]);
+        window_focus(frame->content.clients.buf[selection]->window);
     } else {
         window_unfocus_last();
     }
@@ -1650,10 +1768,10 @@ static void frame_hide(HSFrame* frame) {
     frame_set_visible(frame, false);
     if (frame->type == TYPE_CLIENTS) {
         int i;
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            window_hide(buf[i]);
+            window_hide(buf[i]->window);
         }
     }
 }
@@ -1666,10 +1784,10 @@ void frame_hide_recursive(HSFrame* frame) {
 static void frame_show_clients(HSFrame* frame) {
     if (frame->type == TYPE_CLIENTS) {
         int i;
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            window_show(buf[i]);
+            window_show(buf[i]->window);
         }
     }
 }
@@ -1720,13 +1838,13 @@ int frame_remove_command(int argc, char** argv) {
         second = parent->content.layout.a;
     }
     size_t count;
-    Window* wins;
+    HSClient** wins;
     // get all wins from first child
     frame_destroy(first, &wins, &count);
     // and insert them to other child.. inefficiently
     int i;
     for (i = 0; i < count; i++) {
-        frame_insert_window(second, wins[i]);
+        frame_insert_client(second, wins[i]);
     }
     g_free(wins);
     XDestroyWindow(g_display, parent->window);
@@ -1752,8 +1870,10 @@ int frame_remove_command(int argc, char** argv) {
 }
 
 int close_or_remove_command(int argc, char** argv) {
-    if (frame_focused_window(g_cur_frame)) {
-        return window_close_current();
+    HSClient* client = frame_focused_client(g_cur_frame);
+    if (client) {
+        window_close(client->window);
+        return 0;
     } else {
         return frame_remove_command(argc, argv);
     }
@@ -1789,11 +1909,11 @@ int frame_foreach_client(HSFrame* frame, ClientAction action, void* data) {
         }
     } else {
         // frame->type == TYPE_CLIENTS
-        Window* buf = frame->content.clients.buf;
+        HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         HSClient* client;
         for (int i = 0; i < count; i++) {
-            client = get_client_from_window(buf[i]);
+            client = buf[i];
             // do action for each client
             status = action(client, data);
             if (0 != status) {
