@@ -29,6 +29,8 @@ enum {
 enum {
     ALIGN_VERTICAL = 0,
     ALIGN_HORIZONTAL,
+    // temporary values in split_command
+    ALIGN_EXPLODE,
 };
 
 enum {
@@ -72,7 +74,7 @@ typedef struct HSFrame {
     union {
         HSLayout layout;
         struct {
-            Window* buf;
+            struct HSClient** buf;
             size_t  count;
             int     selection;
             int     layout;
@@ -83,7 +85,9 @@ typedef struct HSFrame {
     struct HSTag*   tag;
     struct HSSlice* slice;
     Window window;
+    int    window_transparent;
     bool   window_visible;
+    Rectangle  last_rect; // last rectangle when being drawn
 } HSFrame;
 
 
@@ -97,24 +101,24 @@ void layout_init();
 void layout_destroy();
 // for frames
 HSFrame* frame_create_empty(HSFrame* parent, HSTag* parenttag);
-void frame_insert_window(HSFrame* frame, Window window);
+void frame_insert_client(HSFrame* frame, struct HSClient* client);
 HSFrame* lookup_frame(HSFrame* root, char* path);
 HSFrame* frame_current_selection();
 HSFrame* frame_current_selection_below(HSFrame* frame);
 // finds the subframe of frame that contains the window
-HSFrame* find_frame_with_window(HSFrame* frame, Window window);
+HSFrame* find_frame_with_client(HSFrame* frame, struct HSClient* client);
 // removes window from a frame/subframes
 // returns true, if window was found. else: false
-bool frame_remove_window(HSFrame* frame, Window window);
+bool frame_remove_client(HSFrame* frame, struct HSClient* client);
 // destroys a frame and all its childs
 // then all Windows in it are collected and returned
 // YOU have to g_free the resulting window-buf
-void frame_destroy(HSFrame* frame, Window** buf, size_t* count);
-void frame_split(HSFrame* frame, int align, int fraction);
+void frame_destroy(HSFrame* frame, struct HSClient*** buf, size_t* count);
+bool frame_split(HSFrame* frame, int align, int fraction);
 int frame_split_command(int argc, char** argv, GString* output);
 int frame_change_fraction_command(int argc, char** argv, GString* output);
 
-void frame_apply_layout(HSFrame* frame, XRectangle rect);
+void frame_apply_layout(HSFrame* frame, Rectangle rect);
 void frame_apply_floating_layout(HSFrame* frame, struct HSMonitor* m);
 void frame_update_frame_window_visibility(HSFrame* frame);
 void reset_frame_colors();
@@ -133,6 +137,8 @@ int frame_current_bring(int argc, char** argv, GString* output);
 int frame_current_set_selection(int argc, char** argv);
 int frame_current_cycle_selection(int argc, char** argv);
 int cycle_all_command(int argc, char** argv);
+int cycle_frame_command(int argc, char** argv);
+void cycle_frame(int direction, int new_window_index, bool skip_invisible);
 
 void frame_unfocus(); // unfocus currently focused window
 
@@ -142,7 +148,7 @@ HSFrame* frame_neighbour(HSFrame* frame, char direction);
 int frame_inner_neighbour_index(HSFrame* frame, char direction);
 int frame_focus_command(int argc, char** argv, GString* output);
 
-// follow selection to leave and focus this frame
+// follow selection to leaf and focus this frame
 int frame_focus_recursive(HSFrame* frame);
 void frame_do_recursive(HSFrame* frame, void (*action)(HSFrame*), int order);
 void frame_do_recursive_data(HSFrame* frame, void (*action)(HSFrame*,void*),
@@ -154,18 +160,18 @@ int layout_rotate_command();
 // returns success or failure
 int frame_foreach_client(HSFrame* frame, ClientAction action, void* data);
 
-void frame_apply_client_layout_linear(HSFrame* frame, XRectangle rect, bool vertical);
-void frame_apply_client_layout_horizontal(HSFrame* frame, XRectangle rect);
-void frame_apply_client_layout_vertical(HSFrame* frame, XRectangle rect);
+void frame_apply_client_layout_linear(HSFrame* frame, Rectangle rect, bool vertical);
+void frame_apply_client_layout_horizontal(HSFrame* frame, Rectangle rect);
+void frame_apply_client_layout_vertical(HSFrame* frame, Rectangle rect);
 int frame_current_cycle_client_layout(int argc, char** argv, GString* output);
 int frame_current_set_client_layout(int argc, char** argv, GString* output);
 int frame_split_count_to_root(HSFrame* frame, int align);
 
 // returns the Window that is focused
 // returns 0 if there is none
-Window frame_focused_window(HSFrame* frame);
-bool frame_focus_window(HSFrame* frame, Window win);
-bool focus_window(Window win, bool switch_tag, bool switch_monitor);
+struct HSClient* frame_focused_client(HSFrame* frame);
+bool frame_focus_client(HSFrame* frame, struct HSClient* client);
+bool focus_client(struct HSClient* client, bool switch_tag, bool switch_monitor);
 // moves a window to an other frame
 int frame_move_window_command(int argc, char** argv, GString* output);
 /// removes the current frame
