@@ -112,7 +112,8 @@ CommandBinding g_commands[] = {
     CMD_BIND_NO_OUTPUT(   "cycle",          frame_current_cycle_selection),
     CMD_BIND_NO_OUTPUT(   "cycle_all",      cycle_all_command),
     CMD_BIND(             "cycle_layout",   frame_current_cycle_client_layout),
-    CMD_BIND_NO_OUTPUT(   "close",          window_close_current),
+    CMD_BIND_NO_OUTPUT(   "cycle_frame",    cycle_frame_command),
+    CMD_BIND(             "close",          close_command),
     CMD_BIND_NO_OUTPUT(   "close_or_remove",close_or_remove_command),
     CMD_BIND(             "split",          frame_split_command),
     CMD_BIND(             "resize",         frame_change_fraction_command),
@@ -422,7 +423,7 @@ int jumpto_command(int argc, char** argv, GString* output) {
     HSClient* client = NULL;
     string_to_client(argv[1], &client);
     if (client) {
-        focus_window(client->window, true, true);
+        focus_client(client, true, true);
         return 0;
     } else {
         g_string_append_printf(output,
@@ -477,36 +478,30 @@ int unsetenv_command(int argc, char** argv, GString* output) {
 void event_on_configure(XEvent event) {
     XConfigureRequestEvent* cre = &event.xconfigurerequest;
     HSClient* client = get_client_from_window(cre->window);
-    XConfigureEvent ce;
-    ce.type = ConfigureNotify;
-    ce.display = g_display;
-    ce.event = cre->window;
-    ce.window = cre->window;
     if (client) {
         bool changes = false;
-        if (client->sizehints && !client->dragged) {
+        Rectangle newRect = client->last_size;
+        if (client->sizehints_floating &&
+            (is_client_floated(client) || client->pseudotile))
+        {
             cre->width += 2*cre->border_width - 2*client->last_border_width;
             cre->height += 2*cre->border_width - 2*client->last_border_width;
-            if (client->float_size.width != cre->width) changes = true;
-            if (client->float_size.height != cre->height) changes = true;
-            client->float_size.width = cre->width;
-            client->float_size.height = cre->height;
-            ce.x = client->last_size.x;
-            ce.y = client->last_size.y;
-            ce.width = client->last_size.width;
-            ce.height = client->last_size.height;
-            ce.override_redirect = False;
-            ce.border_width = cre->border_width;
-            ce.above = cre->above;
+            if (newRect.width  != cre->width) changes = true;
+            if (newRect.height != cre->height) changes = true;
+            newRect.x = cre->x;
+            newRect.y = cre->y;
+            newRect.width = cre->width;
+            newRect.height = cre->height;
         }
-        if (changes && client->tag->floating) {
+        if (changes && is_client_floated(client)) {
+            client->float_size = newRect;
             client_resize_floating(client, find_monitor_with_tag(client->tag));
         } else if (changes && client->pseudotile) {
+            client->float_size = newRect;
             monitor_apply_layout(find_monitor_with_tag(client->tag));
         } else {
         // FIXME: why send event and not XConfigureWindow or XMoveResizeWindow??
-            XSendEvent(g_display, cre->window, False, StructureNotifyMask,
-                       (XEvent*)&ce);
+            client_send_configure(client);
         }
     } else {
         // if client not known.. then allow configure.
@@ -755,11 +750,11 @@ void buttonpress(XEvent* event) {
     if (mouse_binding_find(be->state, be->button)) {
         mouse_start_drag(event);
     } else {
-        focus_window(be->window, false, true);
-        if (*g_raise_on_click) {
-            HSClient* client = get_client_from_window(be->window);
-            if (client) {
-                client_raise(client);
+        HSClient* client = get_client_from_window(be->window);
+        if (client) {
+            focus_client(client, false, true);
+            if (*g_raise_on_click) {
+                    client_raise(client);
             }
         }
     }
@@ -806,13 +801,13 @@ void enternotify(XEvent* event) {
         HSClient* c = get_client_from_window(ce->window);
         HSFrame* target;
         if (c && c->tag->floating == false
-              && (target = find_frame_with_window(c->tag->frame, ce->window))
+              && (target = find_frame_with_client(c->tag->frame, c))
               && target->content.clients.layout == LAYOUT_MAX
-              && frame_focused_window(target) != ce->window) {
+              && frame_focused_client(target) != c) {
             // don't allow focus_follows_mouse if another window would be
             // hidden during that focus change (which only occurs in max layout)
-        } else {
-            focus_window(ce->window, false, true);
+        } else if (c) {
+            focus_client(c, false, true);
         }
     }
 }
@@ -892,6 +887,9 @@ void propertynotify(XEvent* event) {
                 case XA_WM_HINTS:
                     client_update_wm_hints(client);
                     break;
+                case XA_WM_NORMAL_HINTS:
+                    updatesizehints(client);
+                    break;
                 case XA_WM_NAME:
                     client_update_title(client);
                     break;
