@@ -35,7 +35,7 @@ HSObject*   g_monitor_object;
 HSObject*   g_monitor_by_name_object;
 
 typedef struct RectList {
-    XRectangle rect;
+    Rectangle rect;
     struct RectList* next;
 } RectList;
 
@@ -78,7 +78,7 @@ void monitor_apply_layout(HSMonitor* monitor) {
             return;
         }
         monitor->dirty = false;
-        XRectangle rect = monitor->rect;
+        Rectangle rect = monitor->rect;
         // apply pad
         rect.x += monitor->pad_left;
         rect.width -= (monitor->pad_left + monitor->pad_right);
@@ -156,7 +156,7 @@ int list_padding(int argc, char** argv, GString* output) {
 }
 
 static bool rects_intersect(RectList* m1, RectList* m2) {
-    XRectangle *r1 = &m1->rect, *r2 = &m2->rect;
+    Rectangle *r1 = &m1->rect, *r2 = &m2->rect;
     bool is = TRUE;
     is = is && intervals_intersect(r1->x, r1->x + r1->width,
                                    r2->x, r2->x + r2->width);
@@ -165,8 +165,8 @@ static bool rects_intersect(RectList* m1, RectList* m2) {
     return is;
 }
 
-static XRectangle intersection_area(RectList* m1, RectList* m2) {
-    XRectangle r; // intersection between m1->rect and m2->rect
+static Rectangle intersection_area(RectList* m1, RectList* m2) {
+    Rectangle r; // intersection between m1->rect and m2->rect
     r.x = MAX(m1->rect.x, m2->rect.x);
     r.y = MAX(m1->rect.y, m2->rect.y);
     // the bottom right coordinates of the rects
@@ -193,7 +193,7 @@ static RectList* rectlist_create_simple(int x1, int y1, int x2, int y2) {
 }
 
 static RectList* insert_rect_border(RectList* head,
-                                    XRectangle large, XRectangle center)
+                                    Rectangle large, Rectangle center)
 {
     // given a large rectangle and a center which guaranteed to be a subset of
     // the large rect, the task is to split "large" into pieces and insert them
@@ -223,7 +223,7 @@ static RectList* insert_rect_border(RectList* head,
 }
 
 // insert a new element without any intersections into the given list
-RectList* reclist_insert_disjoint(RectList* head, RectList* element) {
+static RectList* reclist_insert_disjoint(RectList* head, RectList* element) {
     if (!element) {
         return head;
     } else if (!head) {
@@ -235,8 +235,8 @@ RectList* reclist_insert_disjoint(RectList* head, RectList* element) {
         return head;
     } else {
         // element intersects with the head rect
-        XRectangle center = intersection_area(head, element);
-        XRectangle large = head->rect;
+        Rectangle center = intersection_area(head, element);
+        Rectangle large = head->rect;
         head->rect = center;
         head->next = insert_rect_border(head->next, large, center);
         head->next = insert_rect_border(head->next, element->rect, center);
@@ -252,7 +252,7 @@ static void rectlist_free(RectList* head) {
     rectlist_free(next);
 }
 
-static RectList* disjoin_rects(XRectangle* buf, size_t count) {
+static RectList* disjoin_rects(Rectangle* buf, size_t count) {
     RectList* cur;
     struct RectList* rects = NULL;
     for (int i = 0; i < count; i++) {
@@ -269,14 +269,14 @@ int disjoin_rects_command(int argc, char** argv, GString* output) {
     if (argc < 1) {
         return HERBST_NEED_MORE_ARGS;
     }
-    XRectangle* buf = g_new(XRectangle, argc);
+    Rectangle* buf = g_new(Rectangle, argc);
     for (int i = 0; i < argc; i++) {
         buf[i] = parse_rectangle(argv[i]);
     }
 
     RectList* rects = disjoin_rects(buf, argc);
     for (RectList* cur = rects; cur; cur = cur->next) {
-        XRectangle r = cur->rect;
+        Rectangle r = cur->rect;
         g_string_append_printf(output, "%dx%d%+d%+d\n",
             r.width, r.height, r.x, r.y);
     }
@@ -290,7 +290,7 @@ int set_monitor_rects_command(int argc, char** argv, GString* output) {
     if (argc < 1) {
         return HERBST_NEED_MORE_ARGS;
     }
-    XRectangle* templates = g_new0(XRectangle, argc);
+    Rectangle* templates = g_new0(Rectangle, argc);
     for (int i = 0; i < argc; i++) {
         templates[i] = parse_rectangle(argv[i]);
     }
@@ -306,7 +306,7 @@ int set_monitor_rects_command(int argc, char** argv, GString* output) {
     return status;
 }
 
-int set_monitor_rects(XRectangle* templates, size_t count) {
+int set_monitor_rects(Rectangle* templates, size_t count) {
     if (count < 1) {
         return HERBST_INVALID_ARGUMENT;
     }
@@ -386,7 +386,31 @@ static int monitor_attr_index(void* data) {
     return monitor_index_of(m);
 }
 
-HSMonitor* add_monitor(XRectangle rect, HSTag* tag, char* name) {
+static void monitor_attr_tag(void* data, GString* output) {
+    HSMonitor* m = (HSMonitor*) data;
+    g_string_append(output, m->tag->display_name->str);
+}
+
+static void monitor_foreach(void (*action)(HSMonitor*)) {
+    for (int i = 0; i < g_monitors->len; i++) {
+        HSMonitor* m = monitor_with_index(i);
+        action(m);
+    }
+}
+
+static void monitor_unlink_id_object(HSMonitor* m) {
+    hsobject_unlink(g_monitor_object, &m->object);
+}
+
+static void monitor_link_id_object(HSMonitor* m) {
+    GString* index_str = g_string_new("");
+    int index = monitor_index_of(m);
+    g_string_printf(index_str, "%d", index);
+    hsobject_link(g_monitor_object, &m->object, index_str->str);
+    g_string_free(index_str, true);
+}
+
+HSMonitor* add_monitor(Rectangle rect, HSTag* tag, char* name) {
     assert(tag != NULL);
     HSMonitor* m = g_new0(HSMonitor, 1);
     hsobject_init(&m->object);
@@ -409,6 +433,7 @@ HSMonitor* add_monitor(XRectangle rect, HSTag* tag, char* name) {
     HSAttribute attributes[] = {
         ATTRIBUTE_STRING(   "name",     m->display_name,ATTR_READ_ONLY  ),
         ATTRIBUTE_CUSTOM_INT("index",   monitor_attr_index,ATTR_READ_ONLY  ),
+        ATTRIBUTE_CUSTOM(   "tag",      monitor_attr_tag,ATTR_READ_ONLY  ),
         ATTRIBUTE_BOOL(     "lock_tag", m->lock_tag,    ATTR_READ_ONLY  ),
         ATTRIBUTE_LAST,
     };
@@ -416,6 +441,8 @@ HSMonitor* add_monitor(XRectangle rect, HSTag* tag, char* name) {
 
     stack_insert_slice(g_monitor_stack, m->slice);
     g_array_append_val(g_monitors, m);
+    monitor_link_id_object(m);
+
     return g_array_index(g_monitors, HSMonitor*, g_monitors->len-1);
 }
 
@@ -424,7 +451,7 @@ int add_monitor_command(int argc, char** argv, GString* output) {
     if (argc < 2) {
         return HERBST_NEED_MORE_ARGS;
     }
-    XRectangle rect = parse_rectangle(argv[1]);
+    Rectangle rect = parse_rectangle(argv[1]);
     HSTag* tag = NULL;
     char* name = NULL;
     if (argc == 2 || !strcmp("", argv[2])) {
@@ -524,8 +551,10 @@ int remove_monitor(int index) {
         g_string_free(monitor->name, true);
     }
     g_string_free(monitor->display_name, true);
-    g_free(monitor);
+    monitor_foreach(monitor_unlink_id_object);
     g_array_remove_index(g_monitors, index);
+    g_free(monitor);
+    monitor_foreach(monitor_link_id_object);
     if (g_cur_monitor >= g_monitors->len) {
         g_cur_monitor--;
         // if selection has changed, then relayout focused monitor
@@ -550,7 +579,7 @@ int move_monitor_command(int argc, char** argv, GString* output) {
             "%s: Monitor \"%s\" not found!\n", argv[0], argv[1]);
         return HERBST_INVALID_ARGUMENT;
     }
-    XRectangle rect = parse_rectangle(argv[2]);
+    Rectangle rect = parse_rectangle(argv[2]);
     if (rect.width < WINDOW_MIN_WIDTH || rect.height < WINDOW_MIN_HEIGHT) {
         g_string_append_printf(output,
             "%s: Rectangle is too small\n", argv[0]);
@@ -640,7 +669,7 @@ int monitor_rect_command(int argc, char** argv, GString* output) {
     } else {
         m = get_current_monitor();
     }
-    XRectangle rect = m->rect;
+    Rectangle rect = m->rect;
     if (with_pad) {
         rect.x += m->pad_left;
         rect.width -= m->pad_left + m->pad_right;
@@ -687,7 +716,7 @@ void ensure_monitors_are_available() {
         return;
     }
     // add monitor if necessary
-    XRectangle rect = {
+    Rectangle rect = {
         .x = 0, .y = 0,
         .width = DisplayWidth(g_display, DefaultScreen(g_display)),
         .height = DisplayHeight(g_display, DefaultScreen(g_display)),
@@ -719,10 +748,7 @@ int monitor_count() {
 }
 
 void all_monitors_apply_layout() {
-    for (int i = 0; i < g_monitors->len; i++) {
-        HSMonitor* m = monitor_with_index(i);
-        monitor_apply_layout(m);
-    }
+    monitor_foreach(monitor_apply_layout);
 }
 
 int monitor_set_tag(HSMonitor* monitor, HSTag* tag) {
@@ -1094,10 +1120,10 @@ static bool geom_unique(XineramaScreenInfo *unique, size_t n, XineramaScreenInfo
 }
 
 // inspired by dwm's updategeom()
-bool detect_monitors_xinerama(XRectangle** ret_rects, size_t* ret_count) {
+bool detect_monitors_xinerama(Rectangle** ret_rects, size_t* ret_count) {
     int i, j, n;
     XineramaScreenInfo *info, *unique;
-    XRectangle *monitors;
+    Rectangle *monitors;
 
     if (!XineramaIsActive(g_display)) {
         return false;
@@ -1114,7 +1140,7 @@ bool detect_monitors_xinerama(XRectangle** ret_rects, size_t* ret_count) {
     XFree(info);
     n = j;
 
-    monitors = g_new(XRectangle, n);
+    monitors = g_new(Rectangle, n);
     for (i = 0; i < n; i++) {
         monitors[i].x = unique[i].x_org;
         monitors[i].y = unique[i].y_org;
@@ -1128,16 +1154,16 @@ bool detect_monitors_xinerama(XRectangle** ret_rects, size_t* ret_count) {
 }
 #else  /* XINERAMA */
 
-bool detect_monitors_xinerama(XRectangle** ret_rects, size_t* ret_count) {
+bool detect_monitors_xinerama(Rectangle** ret_rects, size_t* ret_count) {
     return false;
 }
 
 #endif /* XINERAMA */
 
 // monitor detection that always works: one monitor across the entire screen
-bool detect_monitors_simple(XRectangle** ret_rects, size_t* ret_count) {
+bool detect_monitors_simple(Rectangle** ret_rects, size_t* ret_count) {
     *ret_count = 1;
-    *ret_rects = g_new0(XRectangle, 1);
+    *ret_rects = g_new0(Rectangle, 1);
     (*ret_rects)->x = 0;
     (*ret_rects)->y = 0;
     (*ret_rects)->width = g_screen_width;
@@ -1150,7 +1176,7 @@ int detect_monitors_command(int argc, char **argv, GString* output) {
         detect_monitors_xinerama,
         detect_monitors_simple,
     };
-    XRectangle* monitors = NULL;
+    Rectangle* monitors = NULL;
     size_t count = 0;
     // search for a working monitor detection
     // at least the simple detection must work
@@ -1208,8 +1234,7 @@ void monitor_restack(HSMonitor* monitor) {
     buf[0] = monitor->stacking_window;
     stack_to_window_buf(monitor->tag->stack, buf + 1, count - 1, false, NULL);
     /* remove a focused fullscreen client */
-    Window win = frame_focused_window(monitor->tag->frame);
-    HSClient* client = win ? get_client_from_window(win) : NULL;
+    HSClient* client = frame_focused_client(monitor->tag->frame);
     if (client && client->fullscreen) {
         XRaiseWindow(g_display, client->window);
         int idx = array_find(buf, count, sizeof(*buf), &client->window);
