@@ -1,4 +1,4 @@
-/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -8,7 +8,7 @@
 
 #include <stdbool.h>
 #include <regex.h>
-#include <glib.h>
+#include "glib-backports.h"
 
 struct HSClient;
 struct HSTag;
@@ -29,7 +29,10 @@ typedef struct {
     bool negated;
     union {
         char*       str;
-        regex_t     exp;
+        struct {
+            regex_t     exp;
+            char*       str;
+        } reg;
         int         integer;
     } value;
 } HSCondition;
@@ -43,6 +46,7 @@ typedef struct {
 } HSConsequence;
 
 typedef struct {
+    char*           label;
     HSCondition**   conditions;
     int             condition_count;
     HSConsequence** consequences;
@@ -71,8 +75,12 @@ void client_changes_free_members(HSClientChanges* changes);
 HSRule* rule_create();
 void rule_destroy(HSRule* rule);
 
+void rule_complete(int argc, char** argv, int pos, GString* output);
+
 int rule_add_command(int argc, char** argv, GString* output);
 int rule_remove_command(int argc, char** argv, GString* output);
+int rule_print_all_command(int argc, char** argv, GString* output);
+void complete_against_rule_names(int argc, char** argv, int pos, GString* output);
 
 #endif
 
