@@ -51,6 +51,7 @@ SettingsPair g_settings[] = {
     SET_STRING( "frame_bg_normal_color",           "black",     FR_COLORS   ),
     SET_STRING( "frame_bg_active_color",           "black",     FR_COLORS   ),
     SET_INT(    "frame_bg_transparent",            0,           FR_COLORS   ),
+    SET_INT(    "frame_transparent_width",         0,           FR_COLORS   ),
     SET_INT(    "frame_border_width",              2,           FR_COLORS   ),
     SET_INT(    "frame_border_inner_width",        0,           FR_COLORS   ),
     SET_INT(    "frame_active_opacity",            100,         FR_COLORS   ),
@@ -75,6 +76,7 @@ SettingsPair g_settings[] = {
     SET_INT(    "smart_window_surroundings",       0,           RELAYOUT    ),
     SET_INT(    "monitors_locked",                 0,           LOCK_CHANGED),
     SET_INT(    "auto_detect_monitors",            0,           NULL        ),
+    SET_INT(    "pseudotile_center_threshold",    20,          RELAYOUT    ),
     SET_STRING( "tree_style",                      "*| +`--.",  FR_COLORS   ),
     SET_STRING( "wmname",                  WINDOW_MANAGER_NAME, WMNAME      ),
 };
@@ -211,7 +213,7 @@ int settings_toggle(int argc, char** argv, GString* output) {
     }
     return 0;
 }
-bool memberequals_settingspair(void* pmember, void* needle) {
+static bool memberequals_settingspair(void* pmember, void* needle) {
     char* str = *(char**)pmember;
     SettingsPair* pair = needle;
     if (pair->type == HS_Int) {
