@@ -1,4 +1,4 @@
-/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -6,7 +6,7 @@
 #ifndef __HERBSTLUFT_SETTINGS_H_
 #define __HERBSTLUFT_SETTINGS_H_
 
-#include <glib.h>
+#include "glib-backports.h"
 
 enum {
     HS_String = 0,
