@@ -61,10 +61,10 @@ void stack_insert_slice(HSStack* s, HSSlice* elem);
 void stack_remove_slice(HSStack* s, HSSlice* elem);
 void stack_raise_slide(HSStack* stack, HSSlice* slice);
 void stack_mark_dirty(HSStack* s);
-void stack_slice_add_layer(HSStack* s, HSSlice* slice, HSLayer layer);
-void stack_slice_remove_layer(HSStack* s, HSSlice* slice, HSLayer layer);
+void stack_slice_add_layer(HSStack* stack, HSSlice* slice, HSLayer layer);
+void stack_slice_remove_layer(HSStack* stack, HSSlice* slice, HSLayer layer);
 bool stack_is_layer_empty(HSStack* s, HSLayer layer);
-void stack_clear_layer(HSStack* s, HSLayer layer);
+void stack_clear_layer(HSStack* stack, HSLayer layer);
 
 int print_stack_command(int argc, char** argv, GString* output);
 
