@@ -65,13 +65,20 @@ char** argv_duplicate(int argc, char** argv);
 // frees all entries in argument-vector and then the vector itself
 void argv_free(int argc, char** argv);
 
-XRectangle parse_rectangle(char* string);
+typedef struct {
+    int x;
+    int y;
+    int width;
+    int height;
+} Rectangle;
+
+Rectangle parse_rectangle(char* string);
 
 void g_queue_remove_element(GQueue* queue, GList* elem);
 
 // find an element in an array buf with elems elements of size size.
 int array_find(void* buf, size_t elems, size_t size, void* needle);
-void array_reverse(void* buf, size_t elems, size_t size);
+void array_reverse(void* void_buf, size_t elems, size_t size);
 
 int min(int a, int b);
 
